"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderClient = void 0;
const logger_1 = require("./logger");
const renderClient = (authToken) => {
    const host = `rndr.x.io`;
    return {
        fetchCompletedJobs(limit, finishedAfter, finishedBefore, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const r = {
                    'record_limit': String(limit)
                };
                if (finishedAfter) {
                    // .split('T')[0]
                    r['finished_after'] = finishedAfter.toISOString();
                }
                if (finishedBefore) {
                    r['finished_before'] = finishedBefore.toISOString();
                }
                const params = new URLSearchParams(r);
                const url = `https://${host}/api/v3/services/solana/burns?${params.toString()}`;
                log.info(`fetching ${url}`);
                const res = yield fetch(url, {
                    headers: {
                        Authorization: `${authToken}`,
                        Accept: 'application/vnd.api+json'
                    }
                }).then((res) => __awaiter(this, void 0, void 0, function* () {
                    if (res.status != 200) {
                        throw new Error(`failed fetching node operator rewards. code: ${res.status} body: ${JSON.stringify(yield res.text())}`);
                    }
                    return (yield res.json());
                }));
                return res.data.map(j => ({
                    jobId: Number(j.id),
                    finishedAt: new Date(Date.parse(j.attributes.finishedAt)),
                    rndrUsed: Number(j.attributes.rndrUsed)
                }));
            });
        },
        fetchNodeOperatorRewards(startTime, endTime, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const startTimeStr = startTime.toISOString().split('T')[0];
                const endTimeStr = endTime.toISOString().split('T')[0];
                const url = `https://${host}/api/v3/services/solana/payments?finished_after=${startTimeStr}&finished_before=${endTimeStr}`;
                log.info(`fetching ${url}`);
                const res = yield fetch(url, {
                    headers: {
                        Authorization: `${authToken}`,
                        Accept: 'application/vnd.api+json'
                    }
                }).then((res) => __awaiter(this, void 0, void 0, function* () {
                    if (res.status != 200) {
                        throw new Error(`failed fetching node operator rewards. code: ${res.status} body: ${JSON.stringify(yield res.text())}`);
                    }
                    return (yield res.json());
                }));
                const solKeyToDue = {};
                for (const r of res.data.attributes.paymentsOwed) {
                    const [addr, amt] = Object.entries(r)[0];
                    solKeyToDue[addr] = amt;
                }
                return { solKeyToDue, jobIds: res.meta.jobIds };
            });
        },
        markJobsPayed(ids, finishedAfter, finishedBefore, totalDistributed, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const payload = {
                    data: {
                        attributes: {
                            finished_after: finishedAfter.toISOString().split('T')[0],
                            finished_before: finishedBefore.toISOString().split('T')[0],
                            job_ids: ids,
                            total_distributed: totalDistributed
                        },
                        type: 'solana_payments'
                    }
                };
                const res = yield fetch(`https://${host}/api/v3/services/solana/payments`, {
                    method: 'POST',
                    headers: {
                        'Authorization': `Bearer ${authToken}`,
                        'Content-Type': `application/vnd.api+json`,
                        'Accept': `application/vnd.api+json`
                    },
                    body: JSON.stringify(payload)
                }).then((res) => __awaiter(this, void 0, void 0, function* () {
                    if (res.status != 201) {
                        throw new Error(`failed posting node operator rewards. code: ${res.status} body: ${JSON.stringify(yield res.text())}`);
                    }
                    return (yield res.text());
                }));
                log.info(`markJobsPayed ${res}`);
                return;
            });
        },
        markJobsBurned(ids, totalBurned, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const payload = {
                    data: {
                        attributes: {
                            job_ids: ids,
                            total_burnt: totalBurned
                        },
                        type: 'solana_burns'
                    }
                };
                log.info(`markJobsBurned marking ${ids.length} jobs. burned ${totalBurned}`);
                const res = yield fetch(`https://${host}/api/v3/services/solana/burns`, {
                    method: 'POST',
                    headers: {
                        'Authorization': `Bearer ${authToken}`,
                        'Content-Type': `application/vnd.api+json`,
                        'Accept': `application/vnd.api+json`
                    },
                    body: JSON.stringify(payload)
                }).then((res) => __awaiter(this, void 0, void 0, function* () {
                    if (res.status == 404) {
                        log.info(`already marked burned`);
                    }
                    else if (res.status != 201) {
                        throw new Error(`failed posting node operator rewards. code: ${res.status} body: ${JSON.stringify(yield res.text())}`);
                    }
                    return (yield res.text());
                }));
                log.info(`markJobsBurned success: ${res}`);
                return;
            });
        },
    };
};
exports.renderClient = renderClient;
//# sourceMappingURL=renderClient.js.map