import { consoleLogger } from "./logger";
export const renderClient = (authToken) => {
    const host = `rndr.x.io`;
    return {
        async fetchCompletedJobs(limit, finishedAfter, finishedBefore, log = consoleLogger) {
            const r = {
                'record_limit': String(limit)
            };
            if (finishedAfter) {
                // .split('T')[0]
                r['finished_after'] = finishedAfter.toISOString();
            }
            if (finishedBefore) {
                r['finished_before'] = finishedBefore.toISOString();
            }
            const params = new URLSearchParams(r);
            const url = `https://${host}/api/v3/services/solana/burns?${params.toString()}`;
            log.info(`fetching ${url}`);
            const res = await fetch(url, {
                headers: {
                    Authorization: `${authToken}`,
                    Accept: 'application/vnd.api+json'
                }
            }).then(async (res) => {
                if (res.status != 200) {
                    throw new Error(`failed fetching node operator rewards. code: ${res.status} body: ${JSON.stringify(await res.text())}`);
                }
                return (await res.json());
            });
            return res.data.map(j => ({
                jobId: Number(j.id),
                finishedAt: new Date(Date.parse(j.attributes.finishedAt)),
                rndrUsed: Number(j.attributes.rndrUsed)
            }));
        },
        async fetchNodeOperatorRewards(startTime, endTime, log = consoleLogger) {
            const startTimeStr = startTime.toISOString().split('T')[0];
            const endTimeStr = endTime.toISOString().split('T')[0];
            const url = `https://${host}/api/v3/services/solana/payments?finished_after=${startTimeStr}&finished_before=${endTimeStr}`;
            log.info(`fetching ${url}`);
            const res = await fetch(url, {
                headers: {
                    Authorization: `${authToken}`,
                    Accept: 'application/vnd.api+json'
                }
            }).then(async (res) => {
                if (res.status != 200) {
                    throw new Error(`failed fetching node operator rewards. code: ${res.status} body: ${JSON.stringify(await res.text())}`);
                }
                return (await res.json());
            });
            const solKeyToDue = {};
            for (const r of res.data.attributes.paymentsOwed) {
                const [addr, amt] = Object.entries(r)[0];
                solKeyToDue[addr] = amt;
            }
            return { solKeyToDue, jobIds: res.meta.jobIds };
        },
        async markJobsPayed(ids, finishedAfter, finishedBefore, totalDistributed, log = consoleLogger) {
            const payload = {
                data: {
                    attributes: {
                        finished_after: finishedAfter.toISOString().split('T')[0],
                        finished_before: finishedBefore.toISOString().split('T')[0],
                        job_ids: ids,
                        total_distributed: totalDistributed
                    },
                    type: 'solana_payments'
                }
            };
            const res = await fetch(`https://${host}/api/v3/services/solana/payments`, {
                method: 'POST',
                headers: {
                    'Authorization': `Bearer ${authToken}`,
                    'Content-Type': `application/vnd.api+json`,
                    'Accept': `application/vnd.api+json`
                },
                body: JSON.stringify(payload)
            }).then(async (res) => {
                if (res.status != 201) {
                    throw new Error(`failed posting node operator rewards. code: ${res.status} body: ${JSON.stringify(await res.text())}`);
                }
                return (await res.text());
            });
            log.info(`markJobsPayed ${res}`);
            return;
        },
        async markJobsBurned(ids, totalBurned, log = consoleLogger) {
            const payload = {
                data: {
                    attributes: {
                        job_ids: ids,
                        total_burnt: totalBurned
                    },
                    type: 'solana_burns'
                }
            };
            log.info(`markJobsBurned marking ${ids.length} jobs. burned ${totalBurned}`);
            const res = await fetch(`https://${host}/api/v3/services/solana/burns`, {
                method: 'POST',
                headers: {
                    'Authorization': `Bearer ${authToken}`,
                    'Content-Type': `application/vnd.api+json`,
                    'Accept': `application/vnd.api+json`
                },
                body: JSON.stringify(payload)
            }).then(async (res) => {
                if (res.status == 404) {
                    log.info(`already marked burned`);
                }
                else if (res.status != 201) {
                    throw new Error(`failed posting node operator rewards. code: ${res.status} body: ${JSON.stringify(await res.text())}`);
                }
                return (await res.text());
            });
            log.info(`markJobsBurned success: ${res}`);
            return;
        },
    };
};
//# sourceMappingURL=renderClient.js.map