import { DEFAULT_MULTISIG_PROGRAM_ID, getAuthorityPDA, getTxPDA } from "@sqds/sdk";
import BN from "bn.js";
import { Transaction, TransactionMessage } from "@solana/web3.js";
import { sendAndConfirmWithRetry } from "@render-foundation/spl-utils";
import { consoleLogger } from "./logger";
import { generated } from "@render-foundation/multisig";
//import {hel} from "@render-foundation/spl-utils/lib/types"
import * as multisig from "@render-foundation/multisig";
import { PROGRAM_ID } from "@render-foundation/multisig";
export const executeSqV4Transaction = async (connection, multisigPda, approvers, instrs, log = consoleLogger) => {
    let multisigAccount = await generated.Multisig.fromAccountAddress(connection, multisigPda);
    let vaultPda = multisig.getVaultPda({
        multisigPda,
        index: 0,
        programId: PROGRAM_ID,
    })[0];
    const transactionIndex = multisig.utils.toBigInt(multisigAccount.transactionIndex) + BigInt(1);
    const txPDA = multisig.getTransactionPda({
        multisigPda,
        index: transactionIndex
    })[0];
    const txMessage = new TransactionMessage({
        payerKey: vaultPda,
        recentBlockhash: (await connection.getLatestBlockhash()).blockhash,
        instructions: instrs
    });
    const createIx = multisig.instructions.vaultTransactionCreate({
        multisigPda,
        transactionIndex,
        creator: approvers[0].publicKey,
        vaultIndex: 0,
        ephemeralSigners: 0,
        transactionMessage: txMessage,
        memo: `execute ${txPDA}`,
        programId: PROGRAM_ID,
    });
    // Create Proposal account.
    const createPropIx = multisig.instructions.proposalCreate({
        multisigPda,
        transactionIndex,
        creator: approvers[0].publicKey,
        programId: multisig.PROGRAM_ID,
    });
    //await connection.confirmTransaction(signature);
    const createTx = new Transaction().add(createIx, createPropIx);
    for (const approver of approvers) {
        createTx.add(multisig.instructions.proposalApprove({
            multisigPda,
            transactionIndex,
            member: approver.publicKey,
            memo: `${approver.publicKey} approve`,
            programId: multisig.PROGRAM_ID,
        }));
    }
    createTx.recentBlockhash = (await connection.getLatestBlockhash()).blockhash;
    createTx.feePayer = approvers[0].publicKey;
    createTx.sign(...approvers);
    await sendAndConfirmWithRetry(connection, createTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    log.info(`created tx ${txPDA}`);
    log.info(`executing tx ${txPDA}`);
    const execTx = await multisig.transactions.vaultTransactionExecute({
        connection: connection,
        feePayer: approvers[0].publicKey,
        blockhash: (await connection.getLatestBlockhash()).blockhash,
        multisigPda,
        transactionIndex,
        member: approvers[0].publicKey,
        programId: PROGRAM_ID
    });
    execTx.sign([approvers[0]]);
    const { txid } = await sendAndConfirmWithRetry(connection, Buffer.from(execTx.serialize()), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    log.info(`executed tx ${txPDA}`);
    return { txSig: txid, txPDA };
};
export const executeSqV3Transaction = async (connection, squadsMsKey, clients, instrs, log = consoleLogger) => {
    const msAuthorityK = getAuthorityPDA(squadsMsKey, new BN(1), DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let txPDA;
    const primClient = clients[0];
    const txIdx = await primClient.getNextTransactionIndex(squadsMsKey);
    txPDA = getTxPDA(squadsMsKey, new BN(txIdx), DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let createTx = new Transaction()
        .add(await primClient.buildCreateTransaction(squadsMsKey, 1, txIdx));
    let instrIdx = 1;
    for (const i of instrs) {
        createTx.add(await primClient.buildAddInstruction(squadsMsKey, txPDA, i, instrIdx));
        instrIdx += 1;
    }
    createTx.add(await primClient.buildActivateTransaction(squadsMsKey, txPDA));
    for (const c of clients) {
        createTx.add(await c.buildApproveTransaction(squadsMsKey, txPDA));
    }
    createTx.recentBlockhash = (await connection.getLatestBlockhash()).blockhash;
    createTx.feePayer = primClient.wallet.publicKey;
    for (const c of clients) {
        createTx = await c.wallet.signTransaction(createTx);
    }
    //createTx.sign(sharedMsKp, authorityKp)
    await sendAndConfirmWithRetry(connection, createTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    log.info(`created tx ${txPDA}`);
    log.info(`executing tx ${txPDA}`);
    let execTx = new Transaction().add(await primClient.buildExecuteTransaction(txPDA, primClient.wallet.publicKey));
    execTx.recentBlockhash = (await connection.getLatestBlockhash()).blockhash;
    execTx.feePayer = primClient.wallet.publicKey;
    execTx = await primClient.wallet.signTransaction(execTx);
    //execTx.sign(args.solFeePayerKp)
    const { txid } = await sendAndConfirmWithRetry(connection, execTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'confirmed');
    log.info(`executed ${txid}`);
    return { txSig: txid, txPDA };
};
//# sourceMappingURL=squads.js.map