import { AddressLookupTableAccount, PublicKey, TransactionInstruction } from "@solana/web3.js";
import { consoleLogger } from "./logger";
export const jupClient = (connection, funder) => {
    const maxRetries = 5;
    return {
        async createTradeInstructions(f, log = consoleLogger) {
            const r = {
                'swapMode': "ExactOut",
                'inputMint': 'EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v',
                'outputMint': 'rndrizKT3MK1iimdxRdWabcF7Zg7AR5T4nud4EkHBof',
                'amount': `${f.renderAmount}`,
                'slippageBps': `${f.slippage ?? 50}`
            };
            if (f.maxAccounts) {
                r['maxAccounts'] = `${f.maxAccounts}`;
            }
            const params = new URLSearchParams(r);
            const fetchQuote = async (retry = 0) => {
                if (retry == maxRetries) {
                    throw new Error(`hard failed fetching jup quote after ${maxRetries}`);
                }
                const res = await fetch(`https://quote-api.jup.ag/v6/quote?${params.toString()}`);
                if (res.status != 200) {
                    log.warn(`failed fetching jup quote on try ${retry}. code: ${res.status} body: ${await res.text()}`);
                    return await fetchQuote(retry + 1);
                }
                return (await res.json());
            };
            const quoteResponse = await fetchQuote();
            log.debug(`renderAmt ${f.renderAmount} quote USDC: ${quoteResponse.inAmount}`);
            const fetchInstructions = async (retry = 0) => {
                if (retry == maxRetries) {
                    throw new Error(`hard failed fetching jup instructions after ${maxRetries}`);
                }
                const res = await fetch('https://quote-api.jup.ag/v6/swap-instructions', {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify({
                        // quoteResponse from /quote api
                        quoteResponse,
                        userPublicKey: funder.publicKey.toBase58(),
                        dynamicComputeUnitLimit: true,
                        // custom priority fee - Jup will calculate
                        prioritizationFeeLamports: 'auto' // or custom lamports: 1000
                        // feeAccount is optional. Use if you want to charge a fee.  feeBps must have been passed in /quote API.
                        // feeAccount: "fee_account_public_key"
                    })
                });
                if (res.status != 200) {
                    log.warn(`failed fetching jup swap instructions on try ${retry}. code: ${res.status} body: ${await res.text()}`);
                    return await fetchInstructions(retry + 1);
                }
                return (await res.json());
            };
            const instructions = await fetchInstructions();
            if (instructions.error) {
                throw new Error("Failed to get swap instructions: " + instructions.error);
            }
            const { 
            //tokenLedgerInstruction, // If you are using `useTokenLedger = true`.
            computeBudgetInstructions, // The necessary instructions to setup the compute budget.
            setupInstructions, // Setup missing ATA for the users.
            swapInstruction: swapInstructionPayload, // The actual swap instruction.
            //cleanupInstruction, // Unwrap the SOL if `wrapAndUnwrapSol = true`.
            addressLookupTableAddresses, // The lookup table addresses that you can use if you are using versioned transaction.
            prioritizationFeeLamports, // jup calced prio fee
             } = instructions;
            const deserializeInstruction = (instruction) => {
                return new TransactionInstruction({
                    programId: new PublicKey(instruction.programId),
                    keys: instruction.accounts.map((key) => ({
                        pubkey: new PublicKey(key.pubkey),
                        isSigner: key.isSigner,
                        isWritable: key.isWritable,
                    })),
                    data: Buffer.from(instruction.data, "base64"),
                });
            };
            const getAddressLookupTableAccounts = async (keys) => {
                const addressLookupTableAccountInfos = await connection.getMultipleAccountsInfo(keys.map((key) => new PublicKey(key)));
                return addressLookupTableAccountInfos.reduce((acc, accountInfo, index) => {
                    const addressLookupTableAddress = keys[index];
                    if (accountInfo) {
                        const addressLookupTableAccount = new AddressLookupTableAccount({
                            key: new PublicKey(addressLookupTableAddress),
                            state: AddressLookupTableAccount.deserialize(accountInfo.data),
                        });
                        acc.push(addressLookupTableAccount);
                    }
                    return acc;
                }, new Array());
            };
            const addressLookupTableAccounts = [];
            addressLookupTableAccounts.push(...(await getAddressLookupTableAccounts(addressLookupTableAddresses)));
            return {
                instructions: [
                    // uncomment if needed:
                    ...setupInstructions.map(deserializeInstruction),
                    ...computeBudgetInstructions.map(deserializeInstruction),
                    deserializeInstruction(swapInstructionPayload),
                    // uncomment if needed: deserializeInstruction(cleanupInstruction),
                ],
                addressLookupTableAccounts,
                quoteAmount: Number(quoteResponse.inAmount)
            };
            /*const {txid} = await sendAndConfirmWithRetry(connection, Buffer.from(transaction.serialize()), {skipPreflight: true}, 'confirmed')
      
            return {txid}*/
        }
    };
};
//# sourceMappingURL=tradeClient.js.map