import { DB, TxStatus } from "./dbTypes";
import { Kysely, Transaction } from "kysely";
import { RenderLogger } from "./logger";
export type { DB } from "./dbTypes";
export type Lock = 'incremental_burn';
export type TransferType = "open_compute_rewards" | "node_operator_rewards" | "upgrade_rewards";
export interface Transfer {
    id?: bigint;
    solKey?: string;
    solTx?: string;
    sqTx?: string;
    txStatus?: TxStatus;
    createdAt?: Date;
    payedAt?: Date;
    amountDue: bigint;
    amountPayed?: bigint;
    operatorRewards?: NodeOperatorRewardsTransfer;
    computeClientRewards?: ComputeClientsRewardsTransfer;
    upgradeRewards?: UpgradeRewardsTransfer;
}
export interface UpgradeRewardsTransfer {
    epochId: number;
    epochStartTime: Date;
    points: bigint;
}
export interface NodeOperatorRewardsTransfer {
    epochId: number;
    startTime: Date;
    endTime: Date;
}
export interface ComputeClientsRewardsTransfer {
    epochId: number;
    deviceId: string;
}
export interface TransactionHandle {
    trx: Transaction<DB>;
    commit: () => void;
    rollback: () => void;
    releaseLock(f: {
        lock: Lock;
        newCursor?: string;
    }): Promise<void>;
}
export interface PgClient {
    db: Kysely<DB>;
    updateComputeClientSolKey(deviceId: string, solKey: string, log?: RenderLogger): Promise<void>;
    getLock(f: {
        lock: Lock;
    }, log?: RenderLogger): Promise<{
        cursor: string;
        acquiredAt: Date;
        releasedAt: Date;
    }>;
    acquireLock(f: {
        lock?: Lock;
    }, log?: RenderLogger): Promise<{
        cursor: string;
        txId: number;
        tx: TransactionHandle;
    }>;
    fetchInfoForEpoch(epochId: bigint, log?: RenderLogger): Promise<{
        jobIds: number[];
        totalRndr: bigint;
        startTime?: Date;
        endTime?: Date;
        burned: bigint;
        burnTx?: string;
    }>;
    payoutTransfersSquads(ts: Record<string, bigint>, payedAt: Date, solTx: string, sqTx: string, log?: RenderLogger): Promise<void>;
    updateSqTx(tx: string, status: TxStatus, log?: RenderLogger): Promise<void>;
    setSqTx(transferIds: bigint[], sqTx: string, log?: RenderLogger): Promise<void>;
    getOutstanding(f: {
        typ: TransferType;
    }, log?: RenderLogger): Promise<Record<string, bigint>>;
    createTransfers(transfers: Transfer[], log?: RenderLogger): Promise<void>;
    fetchTransfers(f: {
        epochId?: number;
        payed?: boolean;
        typ: TransferType;
        createdBefore?: Date;
        status?: TxStatus | "notx";
        sort?: "solKey";
    }, log?: RenderLogger): Promise<Transfer[]>;
    payoutTransferSq(id: bigint, payedAt: Date, solTx: string, amtPayed: bigint, sqTx: string, log?: RenderLogger): Promise<void>;
    payoutTransfer(id: bigint, payedAt: Date, solTx: string, amtPayed: bigint, log?: RenderLogger): Promise<void>;
    payoutTransfers(idToAmt: Record<string, bigint>, payedAt: Date, solTx: string, log?: RenderLogger): Promise<void>;
    getLatestNodeOperatorEpoch(log?: RenderLogger): Promise<{
        epochId: bigint;
        startedAt: Date;
        endedAt: Date;
        savedAt?: Date;
    } | undefined>;
    saveNodeOperatorEpoch(epochId: bigint, transfers: Transfer[], jobIds: number[], totalRndr: bigint, log?: RenderLogger): Promise<void>;
    updateBurned(epochId: bigint, burned: bigint, description: string, burnTx: string, log?: RenderLogger): Promise<void>;
    fetchTransfersByWallet(f: {
        epochId?: number;
        payed?: boolean;
        typ: TransferType;
        address: string;
    }): Promise<Transfer[]>;
    insertBurnedJobs(p: {
        jobs: {
            completedAt: Date;
            jobId: number;
            amount: number;
        }[];
        solTx: string;
        burned: number;
        usdcSpent?: number;
        quoteAmt?: number;
        pricedAt: Date;
        executedAt: Date;
    }, log?: RenderLogger): Promise<void>;
    markJobsBurned(p: {
        solTx: string;
        markedAt: Date;
    }, log?: RenderLogger): Promise<void>;
}
export declare const TRANSFER_TABLE = "transfer";
export declare const NODE_OPERATOR_REWARDS_TRANSFER_TABLE = "node_operator_rewards_transfer";
export declare const OPEN_COMPUTE_REWARDS_TRANSFER_TABLE = "open_compute_rewards_transfer";
export declare const UPGRADE_REWARDS_TRANSFER_TABLE = "upgrade_rewards_transfer";
export declare const SQUADS_TX_TABLE = "squads_tx";
export declare const NODE_OPERATOR_EPOCH_TABLE = "node_operator_epoch";
export declare const JOB_ID_TABLE = "job_id";
export declare const JOB_TABLE = "job";
export declare const BURN_TABLE = "burn";
export declare const LOCK_TABLE = "lock";
export declare const pgClient: (config: {
    database: string;
    host: string;
    user: string;
    password: string;
    port?: number;
    max?: number;
}) => PgClient;
export declare class Deferred<T> {
    #private;
    constructor();
    get promise(): Promise<T>;
    resolve: (value: T | PromiseLike<T>) => void;
    reject: (reason?: any) => void;
}
//# sourceMappingURL=pgClient.d.ts.map